//Find PlayerSpeech in ChallengeVoicePack and link it to c++ code

//************************************
//Precached variables go here

static INT NAMES_CVP_PlayerSpeech = 0;
static FName NAME_CVP_Global;
static FName NAME_CVP_Team;
static FName NAME_CVP_CTFGame;
static FName NAME_CVP_Domination;
static FName NAME_CVP_MonsterHunt;
static DWORD OFFSET_CVP_SendType = 0;

//************************************
//Definitions, classes and events go here

struct APlayerPawn_eventSendVoiceMessageParams
{
	APlayerReplicationInfo* Sender;
	APlayerReplicationInfo* Recipient;
	FName MessageType;
	BYTE MessageID;
	FName BroadcastType;
};

//************************************
//Bind function to native, register uscript function names

static void ReplacePlayerSpeechFunc( UBOOL bSet)
{
	guard(ReplacePlayerSpeechFunc);

	UClass* ChallengeVoicePack = NULL;
	for( TObjectIterator<UClass> It; It; ++It )
		if( It->GetOuter() && (appStricmp( It->GetOuter()->GetName(), TEXT("Botpack") ) == 0) && (appStricmp( It->GetName(), TEXT("ChallengeVoicePack") ) == 0) )
		{
			ChallengeVoicePack = *It;
			break;
		}
	if ( !ChallengeVoicePack )
		return;
	
	UFunction* PlayerSpeech = FindBaseFunction( ChallengeVoicePack, TEXT("PlayerSpeech"));
	if ( !PlayerSpeech ) //Script not serialized?
		return;

	if ( !bSet )
	{
		PlayerSpeech->Func = &UObject::ProcessInternal;
		PlayerSpeech->FunctionFlags &= ~FUNC_Native;
	}
	else
	{
		PlayerSpeech->Func = (Native)&AXC_DummyPawn::PlayerSpeechNative;
		PlayerSpeech->FunctionFlags |= FUNC_Native;
	}
	
	//Register subnames
	if ( !NAMES_CVP_PlayerSpeech )
	{
		NAMES_CVP_PlayerSpeech = 1;
		NAME_CVP_Global = FName(TEXT("Global"),FNAME_Intrinsic);
		NAME_CVP_Team = FName(TEXT("Team"),FNAME_Intrinsic);
		NAME_CVP_CTFGame = FName(TEXT("CTFGame"),FNAME_Intrinsic);
		NAME_CVP_Domination = FName(TEXT("Domination"),FNAME_Intrinsic);
		NAME_CVP_MonsterHunt = FName(TEXT("MonsterHunt"),FNAME_Intrinsic);
		INT Found = 0;
		UProperty* SendTypeProp = FindScriptVariable( ChallengeVoicePack, TEXT("SendType"), &Found);
		check(SendTypeProp);
		OFFSET_CVP_SendType = SendTypeProp->Offset;
	}

	unguard;
}

//************************************
//Utilitary/common methods used by natives


static APlayerReplicationInfo* NearestFriendlySightPawn( APawn* PL, FLOAT MaxCone=512.f)
{
	if ( !PL->PlayerReplicationInfo )
		return NULL;
	APawn* Best = NULL;
	FCoords Coords = GMath.UnitCoords / PL->ViewRotation;
	
	FLOAT BestDist = MaxCone + MaxCone * MaxCone; //Line + Square
	for ( APawn* P=PL->Level->PawnList ; P ; P=P->nextPawn )
	{
		if ( P->bIsPlayer && P->PlayerReplicationInfo && (P->PlayerReplicationInfo->Team == PL->PlayerReplicationInfo->Team) && (P != PL) )
		{
			FVector Delta = P->Location - PL->Location;
			FLOAT Dist = Abs(Delta | Coords.XAxis) + Square(Delta | (Coords.YAxis+Coords.ZAxis)); //Line + Square
			if ( Dist < BestDist )
			{
				BestDist = Dist;
				Best = P;
			}
		}
	}
	return Best ? Best->PlayerReplicationInfo : NULL;
}

//************************************
//Natives go here

//Supposed to run serverside
void AXC_DummyPawn::PlayerSpeechNative( FFrame& Stack, RESULT_DECL)
{
	P_GET_INT( Type);
	P_GET_INT( Index);
	P_GET_INT( Callsign);
	P_FINISH;

	guard( PlayerSpeechNative);

	APlayerPawn* PL = Cast<APlayerPawn>( Owner);
	Type = Clamp( Type, 0, 4);
	//Anti-GPF return
	if ( !PL || !PL->PlayerReplicationInfo )
		return;
	UBOOL bTeamGame = PL->GameReplicationInfo ? PL->GameReplicationInfo->bTeamGame : 0;
	APlayerReplicationInfo* Recipient = NULL;
	FName* ResName = &NAME_CVP_Team;
	
	if ( Type == 0 )
	{
		if ( Callsign == -1 )
			ResName = &NAME_CVP_Global;
	}
	else if ( Type == 2 )
	{
		if ( Index == 2 && Level->Game )
		{
			if ( ClassIsA( Level->Game->GetClass(), NAME_CVP_CTFGame) )
				Index = 10;
			else if ( ClassIsA( Level->Game->GetClass(), NAME_CVP_Domination) || ClassIsA( Level->Game->GetClass(), NAME_CVP_MonsterHunt) )
				Index = 11;
		}
		if ( bTeamGame )
		{
			if ( Callsign >= 0 )
			{
				BYTE Team = PL->PlayerReplicationInfo->Team;
				for ( APawn* P=Level->PawnList; P; P=P->nextPawn )
					if ( P->bIsPlayer && P->PlayerReplicationInfo && (P->PlayerReplicationInfo->TeamID == Callsign) && (P->PlayerReplicationInfo->Team == Team) )
					{
						Recipient = P->PlayerReplicationInfo;
						break;
					}
			}
		}
	}
	else if ( Type == 3 )
		ResName = &NAME_CVP_Global;
	else if ( Type == 4 )
	{
		if ( PL && (Index == 10 || Index == 7 || Index == 15 || Index == 3) )
			Recipient = NearestFriendlySightPawn( PL, 750.f);
	}

	if ( !bTeamGame )
		ResName = &NAME_CVP_Global;  // Not a team game? Send to everyone.

	UFunction* SendVoiceMessageFunc = PL->FindFunction( FName( TEXT("SendVoiceMessage") ));
	if ( !SendVoiceMessageFunc )
		return;
	FName* SendType = (FName*) (((BYTE*)this) + OFFSET_CVP_SendType);

	APlayerPawn_eventSendVoiceMessageParams Params;
	Params.Sender = PL->PlayerReplicationInfo;
	Params.Recipient = Recipient;
	Params.MessageType = SendType[Type];
	Params.MessageID = Index;
	Params.BroadcastType = *ResName;
	PL->ProcessEvent( SendVoiceMessageFunc, &Params);
//	PL.SendVoiceMessage( PL->PlayerReplicationInfo, Recipient, SendType[Type], Index, SendMode );

	unguard;
}
